#!/usr/bin/perl

use Moose;
use Cliente;
use Fornecedor;
use Produto;
use Venda;
use Compra;
use Lucro;
use PessoaFisica;
use PessoaJuridica;
use Switch;
use strict;
use warnings;

use DateTime::Format::Strptime;

my @clientes;
my @fornecedores;
my @produtos;
my @vendas;
my @compras;
my @words;
my $flag;
     
    my $parser = DateTime::Format::Strptime->new(
    pattern => '%d/%m/%Y',
    on_error => 'croak',
    );
    
    
sub LeDadosClientes{
    my ($file1, $flag, @words) = @_;
    my $line = <$file1>;
    
	while($line = <$file1>){
		if ($flag){
		   $flag = 0;
		     }
		else {
			@words = split ";", $line;
		    foreach my $word (@words){}
			my ($codigo,$nome,$endereco,$telefone,$data,$tipo) = @words[0.. 5];
			
			if($tipo eq 'F'){
			
			my $cliente= PessoaFisica->new();
			$cliente->set_Codigo($codigo);
			$cliente->set_Nome($nome);
			$cliente->set_Endereco($endereco);
			$cliente->set_Telefone($telefone);
			$cliente->set_Data($data);
			$cliente->set_Tipo($tipo);
			$cliente->set_CPF($words[6]);
			push(@clientes, $cliente);
			}
		    if($tipo eq 'J'){
			
			my $cliente= PessoaJuridica->new();
			$cliente->set_Codigo($codigo);
			$cliente->set_Nome($nome);
			$cliente->set_Endereco($endereco);
			$cliente->set_Telefone($telefone);
			$cliente->set_Data($data);
			$cliente->set_Tipo($tipo);
			$cliente->set_CNPJ($words[6]);
		    $cliente->set_NumeroInscricao($words[7]);
			push(@clientes, $cliente);
			}
		}
	}
close($file1);
}

sub LeDadosFornecedores{
    my ($file2, $flag, @words) = @_;
    my $line = <$file2>;
	while($line = <$file2>){
		if ($flag){
		   $flag = 0;
		     }
		else {
			@words = split ";", $line;
		    foreach my $word (@words){}

			my $fornecedor= Fornecedor->new();
			$fornecedor->set_Codigo($words[0]);
			$fornecedor->set_Nome($words[1]);
			$fornecedor->set_Endereco($words[2]);
			$fornecedor->set_Telefone($words[3]);
			$fornecedor->set_CNPJ($words[4]);
			$fornecedor->set_PessoaContato($words[5]);
			push(@fornecedores, $fornecedor);

		}
	}
close($file2);
}

sub LeDadosProdutos{
    my ($file3, $flag, @words) = @_;
    my $line = <$file3>;
	while($line = <$file3>){
		if ($flag){
		   $flag = 0;
		     }
		else {
			@words = split ";", $line;
		    foreach my $word (@words){}

			my $produto= Produto->new();

			$produto->set_Codigo($words[0]);
			$produto->set_Descricao($words[1]);
			$produto->set_EstoqueMinimo($words[2]);
			$produto->set_EstoqueAtual($words[3]);
			$produto->set_ValorCusto($words[4]);
			$produto->set_PercentualLucro($words[5]);
			push(@produtos, $produto);

		}
	}
close($file3);
}

sub LeDadosVendas{
    my ($file4, $flag, @words) = @_;
    my $line = <$file4>;
	while($line = <$file4>){
		if ($flag){
		   $flag = 0;
		     }
		else {
			@words = split ";", $line;
		    foreach my $word (@words){}

			my $venda= Venda->new();
			$venda->set_CodigoCliente($words[0]);
			$venda->set_Data($words[1]);
			$venda->set_CodigoProduto($words[2]);
			$venda->set_Quantidade($words[3]);
			$venda->set_ModoPagamento($words[4]);
			push(@vendas, $venda);

		}
	}
close($file4);
}

sub LeDadosCompras{
    my ($file5, $flag, @words) = @_;
    my $line = <$file5>;
	while($line = <$file5>){
		if ($flag){
		   $flag = 0;
		     }
		else {
			@words = split ";", $line;
		    foreach my $word (@words){}

			my $compra= Compra->new();
			$compra->set_NumeroNotaFiscal($words[0]);
			$compra->set_CodigoFornecedor($words[1]);
			$compra->set_Data($parser->parse_datetime($words[2]));
			$compra->set_CodigoProduto($words[3]);
			$compra->set_Quantidade($words[4]);
			push(@compras, $compra);

		}
	}
close($file5);
}


sub RelatorioEstoque{

    my $arquivo = "5-estoque.csv";
	my $quantidadeVendida;
	my $quantidadeComprada;
	my $quantidadeEstoque;
	my $i;
	my $j;
	my $k;
	my $codigo;
	my $descricao;
	
	my @sorted_produtos= sort { $a->get_Descricao cmp $b->get_Descricao } @produtos;

	open(my $buffer, '>', $arquivo) or die "Não foi possível abrir o arquivo $arquivo!";
	print $buffer "Codigo;Produto;Quantidade em Estoque;Observação\n";
	
	for($i=0; $i<=$#produtos; $i++){
	$quantidadeVendida=0;
	$quantidadeComprada=0;
	$quantidadeEstoque=0;
	
	           for($j=0; $j<=$#vendas; $j++){
	           if($sorted_produtos[$i]->get_Codigo eq $vendas[$j]->get_CodigoProduto){
	           $quantidadeVendida=$quantidadeVendida + $vendas[$j]->get_Quantidade;
	           }
	           }
	           
	           for($k=0; $k<=$#compras; $k++){
	           if($sorted_produtos[$i]->get_Codigo eq $compras[$k]->get_CodigoProduto){
	           $quantidadeComprada=$quantidadeComprada + $compras[$k]->get_Quantidade;
	           }
	           }
	           
	           $quantidadeEstoque= $sorted_produtos[$i]->get_EstoqueAtual - $quantidadeVendida + $quantidadeComprada;
	      $codigo=$sorted_produtos[$i]->get_Codigo;
	      $descricao=$sorted_produtos[$i]->get_Descricao;
	print $buffer "$codigo;$descricao;$quantidadeEstoque;";
	
	if($quantidadeEstoque < $sorted_produtos[$i]->get_EstoqueMinimo){
	print $buffer "COMPRE MAIS";
	}
    print $buffer "\n";
    }
}

sub RelatorioAreceber{

    my $arquivo = "2-areceber.csv";
	my $valorTotalAreceber;
	my $i;
	my $j;
	my $k;
	my $valorcusto;
	
	my @sorted_clientes= sort { $a->get_Nome cmp $b->get_Nome } @clientes;

	open(my $buffer, '>', $arquivo) or die "Não foi possível abrir o arquivo $arquivo!";
    print $buffer "Cliente;Tipo;CPF/CNPJ;Telefone;Data de cadastro;Total a pagar\n";
	
	for($i=0; $i<=$#clientes; $i++){
	$valorTotalAreceber=0;
	
	           for($j=0; $j<=$#vendas; $j++){
               
	           if($sorted_clientes[$i]->get_Codigo eq $vendas[$j]->get_CodigoCliente){
	                   for($k=0; $k<=$#produtos; $k++){
	                   if($vendas[$j]->get_CodigoProduto eq $produtos[$k]->get_Codigo){
	                   
	                   $valorcusto= $produtos[$k]->get_ValorCusto;
	                   $valorcusto =~ tr/,/./;
	                   $valorcusto = $valorcusto + 0.0;
	                   
	                   $valorTotalAreceber=$valorTotalAreceber +  $vendas[$j]->get_Quantidade*($valorcusto*(1 + $produtos[$k]->get_PercentualLucro/100));
	                   }
	                   }
	           }
	           }
	 
	 my $nome = $sorted_clientes[$i]->get_Nome;
	 my $tipo = $sorted_clientes[$i]->get_Tipo;
	 $valorTotalAreceber=sprintf('%.2f', $valorTotalAreceber);
	 
	           
	 print $buffer "$nome;$tipo;";
	 
	 if($sorted_clientes[$i]->get_Tipo eq 'F'){
	 my $CPF = $sorted_clientes[$i]->get_CPF;
	 print $buffer "$CPF;";
	 }
	 if($sorted_clientes[$i]->get_Tipo eq 'J'){
	 my $CNPJ = $sorted_clientes[$i]->get_CNPJ;
	 print $buffer "$CNPJ;";
	 }
	 my $telefone = $sorted_clientes[$i]->get_Telefone;;
	 my $data= $sorted_clientes[$i]->get_Data;
	 print $buffer "$telefone;$data; $valorTotalAreceber\n";
	 }
}

#die "Erro de especificação dos arquivos. Use: -c <clientes> -f <fornecedores> -p <produtos> -v <vendas> -a <compras>", "\n" unless @ARGV == 4;
my ($parametro1, $f1, $parametro2, $f2, $parametro3, $f3, $parametro4, $f4, $parametro5, $f5) = @ARGV;


my ($file1, $file2, $file3, $file4, $file5);

my $i=0;
while($i<10){
                   switch($ARGV[$i]){
                   
                   case '-c'{
                   open ($file1, '<', $ARGV[$i+1]) or die "Não foi possível abrir o arquivo '$file1','\n  $!";
                   &LeDadosClientes($file1, $flag, @words);
                   }
                   case '-f'{
                   open ($file2, '<', $ARGV[$i+1]) or die "Não foi possível abrir o arquivo '$file2','\n  $!";
                   &LeDadosFornecedores($file2, $flag, @words);
                   }
                   case '-p'{
                   open ($file3, '<', $ARGV[$i+1]) or die "Não foi possível abrir o arquivo '$file3','\n  $!";
                   &LeDadosProdutos($file3, $flag, @words);
                   }
                   case '-a'{
                   open ($file4, '<', $ARGV[$i+1]) or die "Não foi possível abrir o arquivo '$file4','\n  $!";
                   &LeDadosCompras($file4, $flag, @words);
                   }
                   case '-v'{
                   open ($file5, '<', $ARGV[$i+1]) or die "Não foi possível abrir o arquivo '$file5','\n  $!";
                   &LeDadosVendas($file5, $flag, @words);
                   }
            }
            $i=$i+2;
     }
     
&RelatorioEstoque;
&RelatorioAreceber;
     
1;
